// definitions of member functions from matrix.h

namespace mathlib {

template<int R, int C> template<class T> inline
matrix<R, C>::matrix(const tmatrix_dim<R, C, T> &m) {
	typedef typename assign_dispatch<matrix<R, C>, T>::handler handler;
	handler::go(*this, m);
}

template<int R, int C> template<class T> inline
matrix<R, C>::matrix(const tmatrix_ndim<T> &m) {
	typedef typename assign_dispatch<matrix<R, C>, T>::handler handler;
	handler::go(*this, m);
}

template<int R, int C, class B> template<class T> inline
tmatrix_dim<R, C, B> &
tmatrix_dim<R, C, B>::operator=(const tmatrix_dim<R, C, T> &m)
{
	typedef typename assign_dispatch<B, T>::handler handler;
	handler::go(*this, m);
	return *this;
}

template<int R, int C, class B> template<class T> inline
tmatrix_dim<R, C, B> &tmatrix_dim<R, C, B>::operator=(const tmatrix_ndim<T> &m)
{
	typedef typename assign_dispatch<B, T>::handler handler;
	handler::go(*this, m);
	return *this;
}

template<int R, int C, class B> inline
tmatrix_dim<R, C, B> &tmatrix_dim<R, C, B>::operator=(zero_t)
{
	typedef typename assign_dispatch_elop<unop_copy, B, 
		zero_assigner_type, R, C, R, C>::handler handler;
	handler::go(*this, zero_assigner_type());
	return *this;
}

template<int R, int C, class B> inline
tmatrix_dim<R, C, B> &tmatrix_dim<R, C, B>::operator=(identity_t)
{
	typedef typename assign_dispatch_elop<unop_copy, B, 
		identity_assigner_type, R, C, R, R>::handler handler;
	handler::go(*this, identity_assigner_type());
	return *this;
}

template<int R, int C> inline
matrix<R, C>::matrix(zero_t)
{
	typedef typename assign_dispatch_elop<unop_copy, basic_matrix<R, C>, 
		zero_assigner_type, R, C, R, C>::handler handler;
	handler::go(*this, zero_assigner_type());
}

template<int R, int C> inline
matrix<R, C>::matrix(identity_t)
{
	typedef typename assign_dispatch_elop<unop_copy, matrix<R, C>, 
		identity_assigner_type, R, C, R, R>::handler handler;
	handler::go(*this, identity_assigner_type());
}

template<int R> inline
vector<R>::vector(zero_t)
{
	typedef typename assign_dispatch_elop<unop_copy, basic_matrix<R, 1>, 
		zero_assigner_type, R, 1, R, 1>::handler handler;
	handler::go(*this, zero_assigner_type());
}

template<int R> inline
vector<R>::vector(float filler)
{
	typedef typename assign_dispatch_elop<unop_copy, vector<R>, 
		float_keeper, R, 1, R, 1>::handler handler;
	handler::go(*this, float_keeper(filler));
}

template<int R> template<class T> inline
vector<R>::vector(const tmatrix_dim<R, 1, T> &v) {
	typedef typename assign_dispatch<vector<R>, T>::handler handler;
	handler::go(*this, v);
}

template<int R> template<class T> inline
vector<R>::vector(const tmatrix_ndim<T> &v) {
	typedef typename assign_dispatch<vector<R>, T>::handler handler;
	handler::go(*this, v);
}

template<int R, int C> template<class T> inline
basic_matrix<R, C>::basic_matrix(const T &m) {
	typedef typename assign_dispatch<mathlib::matrix<R, C>, T>::handler 
		handler;
	handler::go(*this, m);
}

template<int R, int C, class A>
template<class I> inline
tmatrix<matrix_unop<unop_index<I>, A> >
tmatrix_dim<R, C, A>::operator[](matrix_indexer<I>)
	{ return tmatrix<matrix_unop<unop_index<I>, A> >(*this); }

template<int R, int C, class A>
template<class I> inline
const tmatrix<matrix_unop<unop_cindex<I>, A> >
tmatrix_dim<R, C, A>::operator[](matrix_indexer<I>) const
	{ return tmatrix<matrix_unop<unop_cindex<I>, A> >(*this); }

template<int R, int C, class B>
template<class A> inline
tmatrix_dim<R, C, B> &
tmatrix_dim<R, C, B>::operator*=(const tmatrix_dim<C, R, A> &a) {
	matrix<R, C> t = *this * a;
	*this = t;
	return *this;
}

template<int R, int C, class B>
template<class A> inline
tmatrix_dim<R, C, B> &
tmatrix_dim<R, C, B>::operator*=(const tmatrix_dim<R, 1, A> &a) {
	typedef typename assign_op<binop_mul, B, A>::handler handler;
	handler::go(*this, a);
	return *this;
}

template<int R, int C, class B>
template<class A> inline
tmatrix_dim<R, C, B> &
tmatrix_dim<R, C, B>::operator/=(const tmatrix_dim<R, 1, A> &a) {
	typedef typename assign_op<binop_div, B, A>::handler handler;
	handler::go(*this, a);
	return *this;
}

template<int R, int C, class B> inline
tmatrix_dim<R, C, B> &
tmatrix_dim<R, C, B>::operator*=(float a) {
	typedef typename assign_dispatch_elop<binop_mul, B, float_keeper,
		R, C, R, C>::handler handler;
	float_keeper f = a;
	handler::go(*this, f);
	return *this;
}

template<int R, int C, class B> inline
tmatrix_dim<R, C, B> &
tmatrix_dim<R, C, B>::operator/=(float a) {
	typedef typename assign_dispatch_elop<binop_div, B, float_keeper,
		R, C, R, C>::handler handler;
	float_keeper f = a;
	handler::go(*this, f);
	return *this;
}

template<int R, int C, class B>
template<class A> inline
tmatrix_dim<R, C, B> &
tmatrix_dim<R, C, B>::operator+=(const tmatrix_dim<R, C, A> &a) {
	typedef typename assign_op<binop_add, B, A>::handler handler;
	handler::go(*this, a);
	return *this;
}

template<int R, int C, class B>
template<class A> inline
tmatrix_dim<R, C, B> &
tmatrix_dim<R, C, B>::operator-=(const tmatrix_dim<R, C, A> &a) {
	typedef typename assign_op<binop_sub, B, A>::handler handler;
	handler::go(*this, a);
	return *this;
}

} // namespace mathlib
