#ifndef MATHLIB_QUATERNION_H
#define MATHLIB_QUATERNION_H

#include <mathlib/complex.h>

namespace mathlib {

class quaternion {
public:
	float a, b, c, d;

	quaternion() {}
	quaternion(float a_, float b_ = 0.f, float c_ = 0.f, float d_ = 0.f)
		: a(a_), b(b_), c(c_), d(d_) {}

	float real() const { return a; }
	quaternion unreal() const { return quaternion(0.f, b, c, d); }

	quaternion &operator+=(float f) { a += f; return *this; }
	quaternion &operator-=(float f) { a -= f; return *this; }
	quaternion &operator*=(float f)
		{ a *= f; b *= f; c *= f; d *= f; return *this; }
	quaternion &operator/=(float f)
		{ a /= f; b /= f; c /= f; d /= f; return *this; }

	quaternion &operator+=(complex c)
		{ a += c.real(); b += c.imag(); return *this; }
	quaternion &operator-=(complex c)
		{ a -= c.real(); b -= c.imag(); return *this; }
	quaternion &operator*=(complex);
	quaternion &operator/=(complex);

	quaternion &operator+=(const quaternion &q)
		{ a += q.a; b += q.b; c += q.c; d += q.d; return *this; }
	quaternion &operator-=(const quaternion &q)
		{ a -= q.a; b -= q.b; c -= q.c; d -= q.d; return *this; }
	quaternion &operator*=(const quaternion &q)
		{ return *this = *this * q; }
	quaternion &operator/=(const quaternion &q)
		{ return *this = *this / q; }

	friend quaternion operator*(const quaternion &, const quaternion &);
	friend quaternion operator/(const quaternion &, const quaternion &);

	friend float real(const quaternion &q) { return q.real(); }
	friend quaternion unreal(const quaternion &q) { return q.unreal(); }

	friend float abs(const quaternion &q)
		{ return sqrt(q.a*q.a + q.b*q.b + q.c*q.c + q.d*q.d); }
	friend quaternion normalize(const quaternion &q)
		{ return q / abs(q); }
};

} // namespace mathlib

#endif
