#include <mathlib/matrix.tpp>

namespace mathlib {

typedef basic_matrix<3, 3> m3;
typedef basic_matrix<3, 1> v3;

float det_bm(const m3 &m)
{
	return m(0,0) * det(cofactor_bm<0,0>(m))
		- m(1,0) * det(cofactor_bm<1,0>(m))
		+ m(2,0) * det(cofactor_bm<2,0>(m));
}

void matrix_assign_handler::go(m3 &dest, matrix_unop<unop_adjoint, m3> src)
{
	dest(0,0) = +det(cofactor_bm<0,0>(src.a));
	dest(1,0) = -det(cofactor_bm<0,1>(src.a));
	dest(2,0) = +det(cofactor_bm<0,2>(src.a));

	dest(0,1) = -det(cofactor_bm<1,0>(src.a));
	dest(1,1) = +det(cofactor_bm<1,1>(src.a));
	dest(2,1) = -det(cofactor_bm<1,2>(src.a));

	dest(0,2) = +det(cofactor_bm<2,0>(src.a));
	dest(1,2) = -det(cofactor_bm<2,1>(src.a));
	dest(2,2) = +det(cofactor_bm<2,2>(src.a));
}

void matrix_assign_handler::go(m3 &dest, matrix_unop<unop_inverse, m3> src)
{
	float det_src = det_bm(src.a);
	float d = 1.f / det_src;

	// dest = adjoint(src.a) / det(src.a)
	dest(0,0) = d * +det(cofactor_bm<0,0>(src.a));
	dest(1,0) = d * -det(cofactor_bm<0,1>(src.a));
	dest(2,0) = d * +det(cofactor_bm<0,2>(src.a));

	dest(0,1) = d * -det(cofactor_bm<1,0>(src.a));
	dest(1,1) = d * +det(cofactor_bm<1,1>(src.a));
	dest(2,1) = d * -det(cofactor_bm<1,2>(src.a));

	dest(0,2) = d * +det(cofactor_bm<2,0>(src.a));
	dest(1,2) = d * -det(cofactor_bm<2,1>(src.a));
	dest(2,2) = d * +det(cofactor_bm<2,2>(src.a));
}

} // namespace mathlib
